/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:http/http.dart' as http;
import 'package:welltested/src/models/test_request_data.dart';
import 'package:welltested/src/utilities/env_parser.dart';

class GenerationRepository {
  final baseUrl = 'https://api.welltested.ai';

  Future<Map<String, dynamic>> buildUnitTests(
      TestRequestData testRequestData) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => "");
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final response = await http.post(
          Uri.parse('$baseUrl/ai/unit-tests/generate'),
          body: testRequestData.toJson(),
          headers: {"apikey": apiKey});
      if (response.statusCode == 401) {
        throw "🚨 Invalid Welltested API Key. Use the key recieved in the registered mail.";
      } else if (response.statusCode == 500) {
        throw "🚨 Internal Server Error: ${response.body}";
      } else if (response.statusCode != 200) {
        throw SocketException("Server Error");
      } else
        return jsonDecode(response.body);
    } catch (e) {
      throw (e);
    }
  }

  Future<Map<String, dynamic>> updateTest(
      TestRequestData testRequestData, String fixedTest) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => "");
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final response = await http.post(
          Uri.parse('$baseUrl/ai/unit-tests/update'),
          body: testRequestData.toJson()
            ..addEntries([MapEntry("fixed_test_case", fixedTest)]),
          headers: {"apikey": apiKey});
      return jsonDecode(response.body);
    } catch (e) {
      throw (e);
    }
  }
}
