import 'dart:io';

import 'package:args/command_runner.dart';
import 'package:path/path.dart' as p;
import 'package:welltested/cli/operation.dart';
import 'package:welltested/src/utilities/logging.dart';
import 'package:welltested/src/utilities/path_utils.dart';
import 'package:welltested/src/utilities/pubspec_parser.dart';

import 'cache_manager.dart';

class UnitCommand extends Command {
  UnitCommand(String parent) {
    if (parent == 'generate') {
      argParser.addOption('file',
          abbr: 'f',
          help: 'relative path of the file',
          valueHelp: 'bloc/login_bloc.dart');
      argParser.addOption("class",
          abbr: "c",
          help: 'name of the annotated class',
          valueHelp: 'LoginBloc');
      argParser.addOption("method",
          abbr: 'm',
          help: "name of the method under annotated class",
          valueHelp: 'loginWithEmailAndPassword');
    }
  }
  @override
  String get name => 'unit';

  @override
  String get description => 'Subcommand to select unit test';
  Future<void> run() async {
    String currentDirectory = PathUtils.currentPath;
    final pubspecData = await PubspecParser.forPath(currentDirectory);
    await CacheManager().initialize();
    if (parent?.name == 'generate') {
      String? filePath = argResults?['file'];

      // if file path is given, add the current directory to it
      if (filePath != null) {
        filePath = p.join(PathUtils.currentPath, filePath);
        if (!File(filePath).existsSync()) {
          wtLog.error("File does not exist: $filePath");
          return;
        }
      }
      await CLIOperation.generateUnit(
          pubspecData: pubspecData,
          filePath: filePath,
          className: argResults?['class'],
          methodName: argResults?['method']);
      ;
    } else if (parent?.name == 'save') {
      await CLIOperation.saveUnit(pubspecData: pubspecData);
    }
    await CacheManager().closeAndSettle();
  }
}

class GenerateCommmand extends Command {
  @override
  String get name => 'generate';
  @override
  String get description => 'Generate tests for the annotated code';

  GenerateCommmand() {
    addSubcommand(UnitCommand(name));
  }
}

class SaveCommand extends Command {
  @override
  String get name => 'save';

  @override
  String get description => 'Save updates made to existing tests';

  SaveCommand() {
    addSubcommand(UnitCommand(name));
  }
}
